package yrpg.gmscm.rc.rcd.agreement.operator;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.validate.AbstractValidator;
import yrpg.gmscm.rc.rcd.enumerate.CalcmodeEnum;
import yrpg.gmscm.rc.rcd.tools.GetFieldsHelper;


import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.IntStream;



/**
 * @author zhangjiawei
 * &#064;date  2024/4/26 9:43
 */
public class AgmApplyCalcModeSubmitOpPlugin extends AbstractOperationServicePlugIn {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        List<String> fieldKeys = e.getFieldKeys();
        List<String> costApplyFields = GetFieldsHelper.getAllFieldsList("rm_rcd_settle_ask");
        fieldKeys.addAll(costApplyFields);
        fieldKeys.add("rm_rc_policyrecord_dtl.seq");
    }
    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        e.addValidator(new validation()); //注册校验器
        validators = e.getValidators();
    }

    public List<AbstractValidator> validators = null;

    public static class validation extends AbstractValidator {

        @Override
        public void validate() {
            for (ExtendedDataEntity dataEntity : this.dataEntities) {
                DynamicObject billData = dataEntity.getDataEntity();
                //计算方式
                DynamicObject rmCalcmode = billData.getDynamicObject("rm_calcmode");
                String name = rmCalcmode!=null?rmCalcmode.getString("name"):"";
                //判断是否为常规
                if (CalcmodeEnum.COMMON.getKey().equals(name)) {
                    DynamicObjectCollection rcPolicyrecordDtl = billData.getDynamicObjectCollection("rm_rc_policyrecord_dtl");
                    List<DateRange> dateRangeList = new ArrayList<>();
                    if (!rcPolicyrecordDtl.isEmpty()) {
                        for (DynamicObject dynamicObject : rcPolicyrecordDtl) {
                            DynamicObjectCollection dtlDtl = dynamicObject.getDynamicObjectCollection("rm_rc_policyrecord_dtl_dtl");
                            for (DynamicObject row : dtlDtl) {
                                dateRangeList.add(new DateRange(row.getDate("rm_date_begin"),
                                        row.getDate("rm_date_end")));
                            }

                        }
                    }
                    if (hasOverlap(dateRangeList)) {
                        this.addErrorMessage(dataEntity,"物料明细行存在日期范围交叉，请检查后重新提交！");
                    }
                }
            }
        }
        /**
         * 判断一组日期范围是否有重叠。
         * @param dateRanges 一个日期范围的列表，每个日期范围由开始日期和结束日期定义。
         * @return 如果任何两个日期范围有重叠，则返回 true；否则返回 false。
         */
         boolean hasOverlap(List<DateRange> dateRanges) {
            return IntStream.range(0, dateRanges.size())
                    .anyMatch(i -> IntStream.range(i + 1, dateRanges.size())
                            .anyMatch(j -> isOverlap(dateRanges.get(i), dateRanges.get(j))));
        }
        /**
         * 检查两个日期范围是否有重叠。
         * @param dr1 第一个日期范围。
         * @param dr2 第二个日期范围。
         * @return 如果两个日期范围有重叠，则返回 true；否则返回 false。
         */
         boolean isOverlap(DateRange dr1, DateRange dr2) {
            return !(dr1.end.before(dr2.start) || dr2.end.before(dr1.start));
        }


    }
    //静态类 开始时间和结束时间
    private static class DateRange {
        Date start;
        Date end;

        DateRange(Date start, Date end) {
            this.start = start;
            this.end = end;
        }
    }

}
