package yrpg.gmscm.rc.rcd.apply.form;

import cn.romens.cloud.xdepot.algo.RMAlgoQuery;
import cn.romens.cloud.xdepot.entity.base.RMEntityHelper;
import cn.romens.cloud.xdepot.plugin.form.RMAbstractFormPlugin;
import com.alibaba.druid.util.StringUtils;
import com.google.common.collect.Lists;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.algo.RowMeta;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.entity.MainEntityType;
import kd.bos.form.control.EntryGrid;
import kd.bos.form.events.AfterDoOperationEventArgs;
import kd.bos.form.operate.FormOperate;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import yrpg.gmscm.rc.rcd.tools.GetFieldsHelper;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author zhangjiawei
 * 生成返利申请插件
 */
public class WizardNewApplyFormPlugin extends RMAbstractFormPlugin {
    private final Map<String, String> SETTLE_ASK_HEAD = new HashMap<>();
    private final Map<String, String> SETTLE_ASK_ENTRY = new HashMap<>();

    {
        //put(返利结算申请单，结果单),引入弹出赋值到申请单 在此处添加字段对照
        SETTLE_ASK_HEAD.put("rm_sale_org", "rm_sale_org");//销售组织
        SETTLE_ASK_HEAD.put("rm_account_org", "rm_account_org");//核算组织
        SETTLE_ASK_HEAD.put("rm_rc_type", "rm_rc_type");//返利类型
        SETTLE_ASK_HEAD.put("rm_province_area", "rm_province_area");//办事处
        SETTLE_ASK_HEAD.put("yrpg_main_body", "yrpg_main_body");//兑付主体
        SETTLE_ASK_HEAD.put("rm_ticket_business", "yrpg_main_body");//红冲商业

        //put(返利结算申请单，结果单),引入弹出赋值到申请单 在此处添加字段对照
        SETTLE_ASK_ENTRY.put("rm_the_apply_rc", "");//本次返利申请
        SETTLE_ASK_ENTRY.put("rm_customerid", "rm_customer");//终端客户
        SETTLE_ASK_ENTRY.put("rm_dealer", "rm_dealer");//经销商
        SETTLE_ASK_ENTRY.put("rm_materialid", "rm_material");//产品（物料）
        SETTLE_ASK_ENTRY.put("rm_qty", "rm_qty");//数量
        SETTLE_ASK_ENTRY.put("rm_unit_price", "rm_unit_price");//返利系数
        SETTLE_ASK_ENTRY.put("rm_comment", "rm_comment");//备注
        SETTLE_ASK_ENTRY.put("rm_biz_date", "rm_biz_date");//业务日期
        SETTLE_ASK_ENTRY.put("rm_rebate_expense", "rm_rebate_expense");//费用类型
        //追溯使用字段
        SETTLE_ASK_ENTRY.put("rm_src_bill_no", "billno");//来源单据编号
        SETTLE_ASK_ENTRY.put("rm_src_billid", "id");//来源单据ID
        SETTLE_ASK_ENTRY.put("rm_src_bill_rowid", "entity");//来源单据行ID
        SETTLE_ASK_ENTRY.put("rm_core_bill_no", "rm_bus_core_bill_no");//核心单据编号
        SETTLE_ASK_ENTRY.put("rm_core_billid", "rm_bus_core_billid");//核心单据ID
        SETTLE_ASK_ENTRY.put("rm_core_bill_rowid", "rm_bus_core_bill_rowid");//核心单据行ID
    }

    /**
     * 查询触发
     */
    @Override
    public void afterDoOperation(AfterDoOperationEventArgs args) {
        FormOperate formOperate = (FormOperate) args.getSource();
        if (StringUtils.equals("new_apply_rc", formOperate.getOperateKey())) {
            if (args.getOperationResult().isSuccess()) {
                EntryGrid entryGrid = getControl("rm_rcd_wizard_entry");
                int[] selectRows = entryGrid.getSelectRows();//获取当前选中行
                if (selectRows.length == 0) {
                    this.getView().showTipNotification("未选中分录！");
                } else {
                    createBillData(selectRows);
                    //生成返利兑付申请以后关闭界面
                    this.getView().close();
                }
            }
        }
    }

    /**
     * 组装过滤条件
     *
     * @return 过滤条件
     */
    private QFilter getFilter(int[] totalRows) {
        QFilter qFilter = new QFilter("id", QCP.equals, 0);
        for (int totalRow : totalRows) {
            Object billRowId = this.getModel().getValue("rm_src_bill_rowid", totalRow);
            qFilter = qFilter.or("rm_rcd_rc_result_biz.id", QCP.equals, billRowId);
        }
        return qFilter;
    }

    /**
     * 需要查询的字段
     *
     * @param fields        字段标识
     * @param excludeFields 要查询的单据体 默认查询单据头信息
     */
    private String getQueryFields(Set<String> fields, List<String> excludeFields) {
        MainEntityType billType = RMEntityHelper.getEntityType("rm_rcd_rc_result");
        Map<String, IDataEntityProperty> allFields = billType.getAllFields();
        Set<String> resultSet = new HashSet<>();
        for (String field : allFields.keySet()) {
            if (!fields.contains(field)) {
                continue;
            }
            if (allFields.get(field).getParent() != null) {
                String entity = allFields.get(field).getParent().getName();
                if (!"rm_rcd_rc_result".equals(entity)) {
                    if (!excludeFields.contains(entity)) {
                        continue;
                    }
                    resultSet.add(entity + "." + field + " as " + field);
                    resultSet.add(entity + ".id as entity");

                } else {
                    resultSet.add(field);
                }
            }
        }
        resultSet.add("id");
        return String.join(",", resultSet);
    }

    /**
     * 获取结果表的查询数据
     */
    private DataSet getDataSet(int[] totalRows) {
        Set<String> list = new HashSet<>(SETTLE_ASK_HEAD.values());
        list.addAll(SETTLE_ASK_ENTRY.values());
        String rmRcdCalcResult = getQueryFields(list, Lists.newArrayList("rm_rcd_rc_result_biz"));
        //获取过滤条件
        QFilter qFilter = getFilter(totalRows);
        DataSet dataSet = RMAlgoQuery.queryDataSet("rm_rcd_rc_result",
                rmRcdCalcResult,
                qFilter.toArray(),
                null);
        //申请记录查询
        DataSet dataSetApplyLog = RMAlgoQuery.queryDataSet("rm_rcd_rc_result",
                        "rm_rcd_rc_result_record.rm_business_list as rm_business_list ," +
                                "rm_rcd_rc_result_record.rm_type as rm_type," +
                                "rm_rcd_rc_result_record.rm_the_apply_rc as rm_the_apply_rc",
                        qFilter.toArray(),
                        null)
                .distinct()
                .updateField("rm_the_apply_rc", "case when rm_type = 'B' then  rm_the_apply_rc * -1 else rm_the_apply_rc end"); //查询
        dataSetApplyLog = dataSetApplyLog
                .groupBy(new String[]{"rm_business_list"})
                .sum("rm_the_apply_rc")
                .finish();
        RowMeta rowMeta = dataSet.getRowMeta();
        dataSet = dataSet
                .leftJoin(dataSetApplyLog)
                .on("entity", "rm_business_list")
                .select(rowMeta.getFieldNames(), new String[]{"rm_business_list", "rm_the_apply_rc"})
                .finish();
        dataSet = dataSet.updateField("rm_the_apply_rc",
                "case when rm_the_apply_rc = null then 0 else rm_the_apply_rc end  as rm_the_apply_rc");
        return dataSet;
    }

    /**
     * 组装key
     *
     * @param row 数据
     * @return key
     */
    private Object getKey(Row row) {
        //销售组织
        Object rmSaleOrg = row.get("rm_sale_org");
        //组装key
        return rmSaleOrg.toString();
    }

    private void createBillData(int[] totalRows) {
        DataSet dataResults = getDataSet(totalRows);
        QFilter orgDutyFilter = new QFilter("number", QCP.equals, "0801");
        DataSet rmRcRequestBillType = RMAlgoQuery.queryDataSet("rm_rc_requestbill_type", "id", new QFilter[]{orgDutyFilter}, null);
        if (!rmRcRequestBillType.isEmpty()) {
            //单据类型关联
            dataResults = dataResults.addField("Long(" + rmRcRequestBillType.next().getString("id") + ")", "rm_rc_requestbill_type");
        } else {
            dataResults = dataResults.addField("Long(" + 0L + ")", "rm_rc_requestbill_type");

        }
        Map<Object, DynamicObject> srcMap = new HashMap<>();
        Map<Object, BigDecimal> applyRc = getApplyRc(totalRows);

        for (Row row : dataResults) {
            //获取生成维度
            Object key = getKey(row);
            DynamicObject dynamicObject;
            if (srcMap.containsKey(key)) {
                dynamicObject = srcMap.get(key);
                DynamicObjectCollection entryEntity = dynamicObject.getDynamicObjectCollection("entryentity");
                DynamicObject newEntry = entryEntity.addNew();
                addEntry(newEntry, row, applyRc);
            } else {
                dynamicObject = BusinessDataServiceHelper.newDynamicObject("rm_rcd_settle_ask");
                for (String field : SETTLE_ASK_HEAD.keySet()) {
                    dynamicObject.set(field, row.get(SETTLE_ASK_HEAD.get(field)));
                }
                //组织
                Long id = RequestContext.get().getOrgId();
                dynamicObject.set("org", id);
                dynamicObject.set("billstatus", "A");
                //单据类型
                dynamicObject.set("rm_bill_type", row.get("rm_rc_requestbill_type"));
                //获取编码规则单据编号
                String billNo = GetFieldsHelper.getBillNo(dynamicObject);
                //设置单据编码
                dynamicObject.set("billno", billNo);
                dynamicObject.set("rm_if_artificial", false);
                dynamicObject.set("creator", RequestContext.get().getCurrUserId());
                DynamicObjectCollection entryEntity = dynamicObject.getDynamicObjectCollection("entryentity");
                DynamicObject newEntry = entryEntity.addNew();

                addEntry(newEntry, row, applyRc);
            }
            srcMap.put(key, dynamicObject);
        }
        for (Object key : srcMap.keySet()) {
            DynamicObject dynamicObject = srcMap.get(key);
            mergeMaterial(srcMap.get(key));
            srcMap.replace(key, dynamicObject);
        }
        Collection<DynamicObject> values = srcMap.values();
        SaveServiceHelper.save(values.toArray(new DynamicObject[]{}), OperateOption.create());
    }

    /**
     * 获取申请返利
     */
    private Map<Object, BigDecimal> getApplyRc(int[] totalRows) {
        Map<Object, BigDecimal> applyRc = new HashMap<>();
        for (int totalRow : totalRows) {
            Object billRowId = this.getModel().getValue("rm_src_bill_rowid", totalRow);
            BigDecimal rmTheApplyRc = (BigDecimal) this.getModel().getValue("rm_the_apply_rc", totalRow);
            applyRc.put(billRowId, rmTheApplyRc);
        }
        return applyRc;
    }

    /**
     * 单据体赋值
     *
     * @param newEntry 单据
     * @param row      源数据
     */
    private void addEntry(DynamicObject newEntry, Row row, Map<Object, BigDecimal> applyRcMap) {
        for (String field : SETTLE_ASK_ENTRY.keySet()) {


            //本次申请返利
            if ("rm_the_apply_rc".equals(field)) {
                //本次申请返利
                BigDecimal theApplyRc = applyRcMap.get(row.get("entity"));
                newEntry.set(field, theApplyRc);
                continue;
            }
            //未申请返利
            if ("rm_unaftapplyrc".equals(field)) {
                //应付返利
                BigDecimal rmCopeRc = row.getBigDecimal("rm_cope_rc");
                //已申请返利
                BigDecimal afterApplyRc = row.getBigDecimal("rm_the_apply_rc");
                newEntry.set(field, rmCopeRc.subtract(afterApplyRc));
                continue;
            }
            //本次申请无税返利
            if ("rm_no_tax".equals(field)) {
                continue;
            }
            if ("rm_cope_rc".equals(field)) {
                continue;
            }

            newEntry.set(field, row.get(SETTLE_ASK_ENTRY.get(field)));
        }
    }

    /**
     * 汇总返利
     *
     * @param bill 数据
     */
    public void mergeMaterial(DynamicObject bill) {
        DynamicObjectCollection entity = bill.getDynamicObjectCollection("entryentity");
        Map<String, Map<String, Object>> map = new HashMap<>();
        //当前页面的单据体明细
        for (DynamicObject row : entity) {
            Map<String, Object> rowMap = new HashMap<>();

            long rmMaterial = row.getLong("rm_materialid");
            String mapKey = String.valueOf(rmMaterial);
            BigDecimal rmTheApplyRc = row.getBigDecimal("rm_the_apply_rc");
            BigDecimal rmQty = row.getBigDecimal("rm_qty");
            if (map.containsKey(mapKey)) {
                //原来的金额
                Map<String, Object> stringObjectHashMap = map.get(mapKey);
                //现在的金额加上原来的
                stringObjectHashMap.replace("rm_the_apply_rc_a", rmTheApplyRc.add((BigDecimal) stringObjectHashMap.get("rm_the_apply_rc_a")));
                stringObjectHashMap.replace("rm_qty_a", rmQty.add((BigDecimal) stringObjectHashMap.get("rm_qty_a")));
                //替换
                map.replace(mapKey, stringObjectHashMap);
            } else {
                //物料
                rowMap.put("rm_materialid_a", rmMaterial);
                //申请返利金额
                rowMap.put("rm_the_apply_rc_a", rmTheApplyRc);
                //数量
                rowMap.put("rm_qty_a", rmQty);
                map.put(mapKey, rowMap);
            }
        }
        DynamicObjectCollection settleAskGoods = bill.getDynamicObjectCollection("rm_rcd_settle_ask_goods");
        for (String aLong : map.keySet()) {
            DynamicObject entryDynamicObject = settleAskGoods.addNew();
            for (String field : map.get(aLong).keySet()) {
                entryDynamicObject.set(field, map.get(aLong).get(field));
            }
        }
    }

}
