package yrpg.gmscm.rc.rcd.apply.operator;

import cn.romens.cloud.xfactory.exception.RMBizException;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.db.DB;
import kd.bos.db.tx.TX;
import kd.bos.db.tx.TXHandle;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import yrpg.gmscm.rc.rcd.core.DBRoute;
import yrpg.gmscm.rc.rcd.core.orm.IdGenerator;
import yrpg.gmscm.rc.rcd.core.orm.KDIdGenerator;
import yrpg.gmscm.rc.rcd.tools.GetFieldsHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author zhangjiawei
 * @date 2023/8/11 9:54
 */
public class RcApplyAuditOpPlugin extends AbstractOperationServicePlugIn {

    public void onPreparePropertys(PreparePropertysEventArgs e) {
        List<String> costApplyFields = GetFieldsHelper.getAllFieldsList("rm_rcd_settle_ask");
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.addAll(costApplyFields);
    }

    IdGenerator idGenerator = new KDIdGenerator();

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        List<Object> successPkIds = getOperationResult().getSuccessPkIds();
        List<Object[]> updateAuditAfter = new ArrayList<>();
        for (Object successPkId : successPkIds) {
            DynamicObject rmRcdSettleAsk = BusinessDataServiceHelper.loadSingle(successPkId, "rm_rcd_settle_ask");
            boolean ifArtificial = rmRcdSettleAsk.getBoolean("rm_if_artificial");//是否手工新增
            //判断是否是手工录入
            if (!ifArtificial) {
                DynamicObjectCollection detailEntity = rmRcdSettleAsk.getDynamicObjectCollection("entryentity"); //获取页面内 明细单据体
                for (DynamicObject row : detailEntity) {
                    long id = row.getLong("id");
                    Object[] objects = new Object[]{
                            "B",
                            new Date(),
                            id
                    };
                    updateAuditAfter.add(objects);
                }
            } else {
                TXHandle tx = TX.required();
                try {
                    DynamicObject rmRcdCalcResult = BusinessDataServiceHelper.newDynamicObject("rm_rcd_rc_result");
                    Long billId = idGenerator.nextId();
                    //获取编码规则单据编号
                    String billNo = GetFieldsHelper.getBillNo(rmRcdCalcResult);
                    //设置单据编码
                    rmRcdCalcResult.set("org",rmRcdSettleAsk.get("rm_account_org"));
                    rmRcdCalcResult.set("billno",billNo);
                    rmRcdCalcResult.set("id",billId);
                    rmRcdCalcResult.set("billstatus","C");
                    rmRcdCalcResult.set("rm_if_artificial",rmRcdSettleAsk.get("rm_if_artificial"));
                    rmRcdCalcResult.set("rm_remarks",rmRcdSettleAsk.get("rm_note"));
                    DynamicObjectCollection rmRcdSettleAskEntity = rmRcdSettleAsk.getDynamicObjectCollection("entryentity"); //获取页面内 明细单据体
                    for (DynamicObject rmRcdSettleAskEntityRow : rmRcdSettleAskEntity) {
                        //申请单处理
                        Long entryId = idGenerator.nextId();
                        rmRcdSettleAskEntityRow.set("rm_src_bill_rowid",entryId);
                        rmRcdSettleAskEntityRow.set("rm_src_bill_no",billNo);
                        rmRcdSettleAskEntityRow.set("rm_src_billid",billId);
                        //业务清单
                        DynamicObject newResultBizRow = rmRcdCalcResult.getDynamicObjectCollection("rm_rcd_rc_result_biz").addNew();

                        //组合基本信息
                        mergeBasicInfo(rmRcdSettleAsk, rmRcdSettleAskEntityRow, newResultBizRow);
                        //创建日期
                        newResultBizRow.set("rm_biz_date",rmRcdSettleAsk.get("createtime"));
                        //计算日期
                        newResultBizRow.set("rm_calc_date",new Date());
                        //备注
                        newResultBizRow.set("rm_comment",rmRcdSettleAskEntityRow.getLong("rm_comment"));
                        //业务清单信息id
                        newResultBizRow.set("id",entryId);
                        //申请记录
                        DynamicObject newApplyLog = rmRcdCalcResult.getDynamicObjectCollection("rm_rcd_rc_result_record").addNew();
                        newApplyLog.set("rm_apply_date",new Date());//申请时间
                        newApplyLog.set("rm_audit_time",new Date());//审核时间
                        newApplyLog.set("rm_the_apply_rc",rmRcdSettleAskEntityRow.get("rm_the_apply_rc"));//申请金额
                        newApplyLog.set("rm_audit_status","B");//状态
                        newApplyLog.set("rm_type","A");//类型
                        newApplyLog.set("rm_applicant",rmRcdSettleAsk.getDynamicObject("creator"));//申请人
                        newApplyLog.set("rm_src_bill_no",rmRcdSettleAsk.getString("billno"));//来源单据编号
                        newApplyLog.set("rm_src_billid",rmRcdSettleAsk.getLong("id"));//来源单据id
                        newApplyLog.set("rm_src_bill_rowid",rmRcdSettleAskEntityRow.getLong("id"));//来源单据行id
                        newApplyLog.set("rm_business_list",entryId);//业务清单信息id
                        newApplyLog.set("rm_apply_pay_type",rmRcdSettleAsk.getDynamicObject("rm_rc_pay_type"));//支付方式
                        newApplyLog.set("rm_apply_bill_type",rmRcdSettleAsk.getDynamicObject("rm_bill_type"));//单据类型
                    }
                    //保存结果表
                    SaveServiceHelper.save(new DynamicObject[]{rmRcdCalcResult});
                    //更新申请表
                    SaveServiceHelper.save(new DynamicObject[]{rmRcdSettleAsk});
                    tx.commit();
                } catch (Exception exception) {
                    tx.markRollback();
                    throw new RMBizException("下游返利申请审核程序异常，请联系运维人员！");
                } finally {
                    tx.close();
                }
            }
        }
        if (!updateAuditAfter.isEmpty()) {
            DB.executeBatch(DBRoute.rcDbRoute(),
                    "UPDATE rm_rcd_rc_result_record SET frm_audit_status = ?,frm_audit_time = ?  WHERE frm_src_bill_rowid = ?",
                    updateAuditAfter);
        }

    }

    //组合基本信息
    private void mergeBasicInfo(DynamicObject rmRcdSettleAsk, DynamicObject rmRcdSettleAskEntityRow, DynamicObject newResultBizRow) {
        //销售组织
        newResultBizRow.set("rm_account_org", rmRcdSettleAsk.get("rm_account_org"));
        //返利支付方式
        newResultBizRow.set("rm_rc_pay_type", rmRcdSettleAsk.get("rm_rc_pay_type"));
        //销售组织
        newResultBizRow.set("rm_sale_org", rmRcdSettleAsk.get("rm_sale_org"));
        //物料
        newResultBizRow.set("rm_material", rmRcdSettleAskEntityRow.get("rm_materialid"));
        //数量
        newResultBizRow.set("rm_qty", rmRcdSettleAskEntityRow.get("rm_qty"));
        //系数
        newResultBizRow.set("rm_unit_price", rmRcdSettleAskEntityRow.get("rm_unit_price"));
        //应付返利
        newResultBizRow.set("rm_cope_rc", rmRcdSettleAskEntityRow.get("rm_the_apply_rc"));
        //经销商
        newResultBizRow.set("rm_dealer", rmRcdSettleAskEntityRow.get("rm_dealer"));
        //兑付主体 （同经销商）
        newResultBizRow.set("yrpg_main_body", rmRcdSettleAskEntityRow.get("rm_dealer"));
        //终端客户
        newResultBizRow.set("rm_customer", rmRcdSettleAskEntityRow.get("rm_customerid"));
    }
}
