package yrpg.gmscm.rc.rcd.apply.operator;

import cn.romens.cloud.xdepot.algo.RMAlgoQuery;
import kd.bos.algo.DataSet;
import kd.bos.algo.Row;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.db.DB;
import kd.bos.db.tx.TX;
import kd.bos.db.tx.TXHandle;
import kd.bos.dlock.DLock;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.operate.result.OperateErrorInfo;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import org.apache.commons.compress.utils.Lists;
import yrpg.gmscm.rc.rcd.core.DBRoute;
import yrpg.gmscm.rc.rcd.core.orm.IdGenerator;
import yrpg.gmscm.rc.rcd.core.orm.KDIdGenerator;
import yrpg.gmscm.rc.rcd.tools.GetFieldsHelper;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author zhangjiawei
 * @date 2023/8/11 8:48
 */
public class RcApplySubmitOpPlugin extends AbstractOperationServicePlugIn {
    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        List<String> fieldKeys = e.getFieldKeys();
        List<String> costApplyFields = GetFieldsHelper.getAllFieldsList("rm_rcd_settle_ask");
        fieldKeys.addAll(costApplyFields);
        fieldKeys.add("entryentity.seq");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        e.addValidator(new validation()); //注册校验器
        validators = e.getValidators();
    }

    public List<AbstractValidator> validators = null;

    public class validation extends AbstractValidator {
        IdGenerator idGenerator = new KDIdGenerator();

        /**
         * 反写结果表申请记录
         * @param dataEntity
         */
        private void saveApplyLog(ExtendedDataEntity dataEntity) {
            DynamicObject dataEntityData = dataEntity.getDataEntity(); //获取页面数据
            DynamicObjectCollection detailEntity = dataEntityData.getDynamicObjectCollection("entryentity"); //获取页面内 明细单据体
            Set<Long> srcBillIdSet = new HashSet<>();
            Set<Long> srcBillRowIdSet = new HashSet<>();
            for (DynamicObject dynamicObject : detailEntity) {
                srcBillIdSet.add(dynamicObject.getLong("rm_src_billid"));
                srcBillRowIdSet.add(dynamicObject.getLong("rm_src_bill_rowid"));
            }
            QFilter qFilterBusiness = new QFilter("id", QCP.in, srcBillIdSet)
                    .and("rm_rcd_rc_result_biz.id", QCP.in, srcBillRowIdSet);
            String queryBusiness = GetFieldsHelper.getSpecifyEntry("rm_rcd_rc_result", "rm_rcd_rc_result_biz");//获取业务清单单据体字段标识
            DataSet dataBusiness = RMAlgoQuery.queryDataSet("rm_rcd_rc_result", "id," + queryBusiness, qFilterBusiness.toArray(), null);
            Map<Long, Map<String, Object>> coreFieldsMap = new HashMap<>();
            for (Row row : dataBusiness) {
                Map<String, Object> map = new HashMap<>();
                Long entityId = row.getLong("rm_rcd_rc_result_biz.id");
                map.put("coreBillNo", row.getString("rm_bus_core_bill_no"));
                map.put("coreBillid", row.getLong("rm_bus_core_billid"));
                map.put("coreBillRowid", row.getLong("rm_bus_core_bill_rowid"));
                map.put("id", row.getLong("id"));
                coreFieldsMap.put(entityId, map);
            }
            for (DynamicObject row : detailEntity) {
                Long srcBillRowId = row.getLong("rm_src_bill_rowid");//来源单据行id
                Map<String, Object> map = coreFieldsMap.get(srcBillRowId);
                if (map != null) {
                    List<Object> newLog = new ArrayList<>();
                    newLog.add(idGenerator.nextId());//entityid
                    newLog.add(map.get("id"));//id
                    newLog.add(srcBillRowId);//业务清单信息id
                    newLog.add(row.getBigDecimal("rm_the_apply_rc"));//申请金额
                    newLog.add("A");//单据状态
                    newLog.add(new Date());//申请时间
                    newLog.add(RequestContext.get().getCurrUserId());//申请人
                    newLog.add(map.get("coreBillNo"));//核心编号
                    newLog.add(map.get("coreBillid"));//核心单据id
                    newLog.add(map.get("coreBillRowid"));//核心行id
                    newLog.add(dataEntityData.get("billno"));//来源单据编号
                    newLog.add(dataEntityData.get("id"));//来源单据id
                    newLog.add(row.get("id"));//来源单据行id
                    newLog.add("A");//类型
                    newLog.add(dataEntityData.getDynamicObject("rm_rc_pay_type").getPkValue());//支付方式
                    newLog.add(dataEntityData.getDynamicObject("rm_bill_type").getPkValue());//单据类型
                    DB.update(DBRoute.rcDbRoute(),
                            "INSERT INTO rm_rcd_rc_result_record (" +
                                    "fentryid," +
                                    "fid," +
                                    "frm_business_list," +
                                    "frm_the_apply_rc," +
                                    "frm_audit_status," +
                                    "frm_apply_date," +
                                    "frm_applicant," +
                                    "frm_core_bill_no," +
                                    "frm_apply_core_billid," +
                                    "frm_apply_core_bill_rowid," +
                                    "frm_src_bill_no," +
                                    "frm_src_billid," +
                                    "frm_src_bill_rowid," +
                                    "frm_type," +
                                    "frm_apply_pay_type," +
                                    "frm_apply_bill_type" +
                                    ")" +
                                    " VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
                            newLog.toArray(new Object[0]));
                }
            }
        }

        @Override
        public void validate() {
            List<Object> errorIds = Lists.newArrayList();
            for (AbstractValidator validator : validators) {
                List<ExtendedDataEntity> falseDataEntities = validator.getFalseDataEntities();
                for (ExtendedDataEntity falseDataEntity : falseDataEntities) {
                    errorIds.add(falseDataEntity.getBillPkId());
                }
                List<OperateErrorInfo> allErrorInfo = validator.getValidateResult().getAllErrorInfo();
                for (OperateErrorInfo operateErrorInfo : allErrorInfo) {
                    errorIds.add(operateErrorInfo.getPkValue());
                }
            }

            if (this.getOption().getVariables().containsKey("checked")) {
                if (this.getOption().getVariables().containsKey("addWarningMessage")) {
                    for (ExtendedDataEntity dataEntity : this.dataEntities) {
                        saveApplyLog(dataEntity);
                    }
                }
            } else {
                for (ExtendedDataEntity dataEntity : this.dataEntities) {
                    DynamicObject dataEntityData = dataEntity.getDataEntity(); //获取页面数据
                    //界面校验
                    if (errorIds.contains(dataEntityData.getPkValue())) {
                        continue;
                    }
                    //冲红不经过校验
                    if (dataEntityData.getBoolean("rm_is_offset")) {
                        saveApplyLog(dataEntity);
                        continue;
                    }
                    //非手工录入
                    if (!dataEntityData.getBoolean("rm_if_artificial")) {
                        TXHandle tx = TX.required();
                        List<DLock> lockList = Lists.newArrayList();
                        try {
                            //获取页面内 明细单据体
                            DynamicObjectCollection detailEntity = dataEntityData.getDynamicObjectCollection("entryentity");
                            for (DynamicObject row : detailEntity) {
                                Long srcBillId = row.getLong("rm_src_billid");//来源单据id
                                Long srcBillRowId = row.getLong("rm_src_bill_rowid");//来源单据行id
                                QFilter qFilterTheApply = new QFilter("id", QCP.equals, srcBillId)
                                        .and("rm_rcd_rc_result_record.rm_business_list", QCP.equals, srcBillRowId);
                                DataSet dataTheApply = RMAlgoQuery.queryDataSet("rm_rcd_rc_result",
                                        "rm_rcd_rc_result_record.rm_the_apply_rc",
                                        qFilterTheApply.toArray(),
                                        null);
                                BigDecimal aftApplyRc = BigDecimal.ZERO;// 获取已申请返利
                                for (Row rowApplyRc : dataTheApply) {
                                    aftApplyRc = aftApplyRc.add(rowApplyRc.getBigDecimal("rm_rcd_rc_result_record.rm_the_apply_rc"));
                                }
                                BigDecimal theApplyRc = row.getBigDecimal("rm_the_apply_rc");//获取本次申请返利

                                Long billRowid = row.getLong("rm_src_bill_rowid");
                                QFilter qFilter2 = new QFilter("rm_rcd_rc_result_biz.id", QCP.equals, billRowid);
                                BigDecimal copeWithRc = BigDecimal.ZERO;//应付返利
                                DataSet dataSetLog = RMAlgoQuery
                                        .queryDataSet("rm_rcd_rc_result",
                                                "rm_rcd_rc_result_biz.rm_cope_rc",
                                                qFilter2.toArray(),
                                                null); //查询
                                for (Row rowCope : dataSetLog) {
                                    copeWithRc = rowCope.getBigDecimal("rm_rcd_rc_result_biz.rm_cope_rc");
                                }
                                //判断是否同向
                                if (copeWithRc.signum() != theApplyRc.signum() && copeWithRc.signum() != 0) {
                                    String tipsSig = copeWithRc.signum() == -1 ? "本单只能申请负返利" : "本单只能申请正返利";
                                    this.addErrorMessage(dataEntity, tipsSig);
                                    break;
                                }
                                BigDecimal absCopeWithRc = copeWithRc.abs();
                                BigDecimal absAftApplyRc = aftApplyRc.abs();
                                BigDecimal absTheApplyRc = theApplyRc.abs();
                                if ((absCopeWithRc.subtract(absAftApplyRc)).compareTo(absTheApplyRc) < 0) {
                                    BigDecimal tipsApplyRc = copeWithRc.subtract(aftApplyRc);
                                    DecimalFormat df = new DecimalFormat("0.00");
                                    if (tipsApplyRc.compareTo(BigDecimal.ZERO) < 0) {
                                        this.addErrorMessage(dataEntity,
                                                String.format("明细分录%s： 本次申请返利额必须等于大于未申请返利额（" + df.format(tipsApplyRc) + "），请检查是否存在已提交的返利申请！", row.getInt("seq")));
                                    } else {
                                        this.addErrorMessage(dataEntity,
                                                String.format("明细分录%s： 本次申请返利额必须等于小于未申请返利额（" + df.format(tipsApplyRc) + "），请检查是否存在已提交的返利申请！", row.getInt("seq")));
                                    }
                                    break;
                                }
                                DLock lock = DLock.create(srcBillRowId.toString(), "");
                                // 一直等待直到获得锁
                                boolean tryLock = lock.tryLock(10000);
                                if (!tryLock) {
                                    this.addErrorMessage(dataEntity, String.format("返利明细【%s】正在被引用，请重试", srcBillRowId));
                                    break;
                                } else {
                                    lockList.add(lock);
                                }
                                QFilter qFilterBusiness = new QFilter("id", QCP.equals, srcBillId)
                                        .and("rm_rcd_rc_result_biz.id", QCP.equals, srcBillRowId);
                                String queryBusiness = GetFieldsHelper
                                        .getSpecifyEntry("rm_rcd_rc_result",
                                                "rm_rcd_rc_result_biz");//获取业务清单单据体字段标识
                                DataSet dataBusiness = RMAlgoQuery
                                        .queryDataSet("rm_rcd_rc_result",
                                                "id," + queryBusiness,
                                                qFilterBusiness.toArray(),
                                                null);
                                BigDecimal payableApplyRc = BigDecimal.ZERO;//应付
                                Long id = null;//单据id
                                String coreBillNo = "";//核心编码
                                Long coreBillid = 0L;//核心单据id
                                Long coreBillRowid = 0L;//核心单据行id
                                Long entityId = 0L;
                                for (Row datum : dataBusiness) {
                                    payableApplyRc = payableApplyRc.add(datum.getBigDecimal("rm_cope_rc"));
                                    id = datum.getLong("id");
                                    coreBillNo = datum.getString("rm_bus_core_bill_no");
                                    coreBillid = datum.getLong("rm_bus_core_billid");
                                    coreBillRowid = datum.getLong("rm_bus_core_bill_rowid");
                                    entityId = datum.getLong("rm_rcd_rc_result_biz.id");
                                }
                                if (id != null) {
                                    List<Object> newLog = new ArrayList<>();
                                    newLog.add(idGenerator.nextId());//entityid
                                    newLog.add(id);//id
                                    newLog.add(entityId);//业务清单信息id
                                    newLog.add(theApplyRc);//申请金额
                                    newLog.add("A");//单据状态
                                    newLog.add(new Date());//申请时间
                                    newLog.add(RequestContext.get().getCurrUserId());//申请人
                                    newLog.add(coreBillNo);//核心编号
                                    newLog.add(coreBillid);//核心单据id
                                    newLog.add(coreBillRowid);//核心行id
                                    newLog.add(dataEntityData.get("billno"));//来源单据编号
                                    newLog.add(dataEntityData.get("id"));//来源单据id
                                    newLog.add(row.get("id"));//来源单据行id
                                    newLog.add("A");//类型
                                    newLog.add(dataEntityData.getDynamicObject("rm_rc_pay_type").getPkValue());//支付方式
                                    newLog.add(dataEntityData.getDynamicObject("rm_bill_type").getPkValue());//单据类型
                                    DB.update(DBRoute.rcDbRoute(),
                                            "INSERT INTO rm_rcd_rc_result_record (" +
                                                    "fentryid," +
                                                    "fid," +
                                                    "frm_business_list," +
                                                    "frm_the_apply_rc," +
                                                    "frm_audit_status," +
                                                    "frm_apply_date," +
                                                    "frm_applicant," +
                                                    "frm_core_bill_no," +
                                                    "frm_apply_core_billid," +
                                                    "frm_apply_core_bill_rowid," +
                                                    "frm_src_bill_no," +
                                                    "frm_src_billid," +
                                                    "frm_src_bill_rowid," +
                                                    "frm_type," +
                                                    "frm_apply_pay_type," +
                                                    "frm_apply_bill_type" +
                                                    ")" +
                                                    " VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
                                            newLog.toArray(new Object[0]));
                                }
                            }
                            tx.commit();
                        } catch (Exception ex) {
                            tx.markRollback();
                            throw ex;
                        } finally {
                            tx.close();
                            lockList.forEach(DLock::unlock);
                        }

                    }
                }
                this.getOption().setVariableValue("checked", "false");
            }
        }

    }
}
