package yrpg.gmscm.rc.rcd.tools;

import cn.romens.cloud.xdepot.entity.base.RMEntityHelper;
import com.google.common.collect.Lists;
import kd.bos.coderule.api.CodeRuleInfo;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.metadata.IDataEntityProperty;
import kd.bos.entity.MainEntityType;
import kd.bos.servicehelper.coderule.CodeRuleServiceHelper;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author zhangjiawei
 * @date 2022/9/2 15:24
 */
public class GetFieldsHelper {
    /**
     * @param billName 单据标识
     * @param isFormat 是否进行格式化
     * @return 字段标识
     */
    private static Set<String> getFieldsCore(String billName, List<String> includeFields, List<String> excludeFields, boolean isFormat) {
        List<String> fields = new ArrayList<>();
        boolean control = true;
        if (includeFields.size() != 0) {
            control = false;
            fields.addAll(includeFields);
        }else {
            fields.addAll(excludeFields);
        }
        MainEntityType billType = RMEntityHelper.getEntityType(billName);
        Map<String, IDataEntityProperty> allFields = billType.getAllFields();
        Set<String> resultSet = new HashSet<>();
        for (String field : allFields.keySet()) {
            if (excludeFields.contains(field)) {
                continue;
            }
            if (allFields.get(field).getParent() != null) {
                String entity = allFields.get(field).getParent().getName();
                if (fields.contains(entity) == control) {
                    continue;
                }
                if (isFormat) {
                    resultSet.add(field);
                } else {
                    if (!entity.equals(billName)) {
                        resultSet.add(entity + "." + field + " as " + field);
                        resultSet.add(entity + ".id");

                    } else {
                        resultSet.add(field);
                    }
                }
            }
        }
        return resultSet;
    }

    /**
     * 获取单据的字段标识
     *
     * @param billName  单据标识
     * @param skipName  跳过单据体
     * @param skipField 跳过字段标识
     * @return 字段标识
     */
    public static String getFieldsSkipEntityFields(String billName, List<String> skipName, List<String> skipField) {
        skipField.addAll(skipName);
        Set<String> fieldsCore = getFieldsCore(billName, Lists.newArrayList(), skipField, false);
        return String.join(",", fieldsCore);
    }

    /**
     * 获取单据的字段标识
     *
     * @param billName 单据标识
     * @param skipName 跳过单据体
     * @return 字段标识
     */
    public static String getFieldsSkipEntity(String billName, List<String> skipName) {
        return getFieldsSkipEntityFields(billName, skipName, Lists.newArrayList());
    }

    /**
     * 获取单据的所有标识
     *
     * @param billName 单据标识
     * @return 字段标识
     */
    public static String getAllFields(String billName) {
        return getFieldsSkipEntity(billName, new ArrayList<>());
    }

    private static String getAllFields(String billName, String skipName, boolean control) {
        Set<String> fieldsCore;
        if (control) {
            fieldsCore = getFieldsCore(billName, Lists.newArrayList(skipName), Lists.newArrayList(), false);
        } else {
            fieldsCore = getFieldsCore(billName, Lists.newArrayList(), Lists.newArrayList(skipName), false);
        }
        return String.join(",", fieldsCore);
    }


    /**
     * 获取指定单据体的单据字段
     *
     * @param billName  单据标识
     * @param entryName 指定单据体
     * @return 字段标识
     */
    public static String getSpecifyEntry(String billName, String entryName) {
        return getAllFields(billName, entryName, true);
    }

    /**
     * 获取跳过指定单据体的单据字段
     *
     * @param billName  单据标识
     * @param entryName 指定单据体
     * @return 字段标识
     */
    public static String getSpaceEntry(String billName, String entryName) {
        return getAllFields(billName, entryName, false);
    }

    //获取单据实体的字段表示
    public static List<String> getAllFieldsList(String billName, List<String> skipName) {
        Set<String> fieldsCore = getFieldsCore(billName, Lists.newArrayList(),skipName, true);
        return Lists.newArrayList(fieldsCore);
    }

    /**
     * 获取单据所有的字段
     *
     * @param billName 单据标识
     * @return 返回list
     */
    public static List<String> getAllFieldsList(String billName) {
        return getAllFieldsList(billName, Lists.newArrayList());
    }
    public static String getBillNo(DynamicObject newDy){
        CodeRuleInfo codeRule = CodeRuleServiceHelper.getCodeRule(newDy.getDataEntityType().getName(),newDy,null);
        return CodeRuleServiceHelper.getNumber(codeRule, newDy);
    }
}
